import { AppPage } from "./app.po";
import { CalculatorPage } from "./calculator.po";
import { Navbar } from "./navbar.po";
import { SideNav } from "./sidenav.po";
import { browser } from "protractor";
import { PreferencesPage } from "./preferences.po";

/**
 * Load a session containing 3 calculators, having linked parameters
 * from one to another, triggers computation from the top-most, triggers
 * results reset from the bottom-most.
 * Does it once with parameters linked to parameters, once with parameters
 * linked to results.
 */
describe("ngHyd − compute then reset chained results − ", () => {
    let startPage: AppPage;
    let calcPage: CalculatorPage;
    let navbar: Navbar;
    let sidenav: SideNav;
    let prefPage: PreferencesPage;

    function init() {
        startPage = new AppPage();
        calcPage = new CalculatorPage();
        navbar = new Navbar();
        sidenav = new SideNav();
        prefPage = new PreferencesPage();
    }
    beforeEach(init);

    it("when loading session-cascade-params.json, computation should not be chained, but results reset should be", async () => {
        // load session file
        await startPage.navigateTo();
        await navbar.clickMenuButton();
        await browser.sleep(200);
        await sidenav.clickLoadSessionButton();
        await browser.sleep(200);
        await sidenav.loadSessionFile("./session/session-cascade-params.json");
        await browser.sleep(500);
        expect(await navbar.getAllCalculatorTabs().count()).toBe(3);

        // 1. get down-most module
        await navbar.clickCalculatorTabForUid("Y2l2Y3");

        // click "compute" button
        const calcButton = calcPage.getCalculateButton();
        await calcButton.click();

        // down-most module should have results
        let hasResults = await calcPage.hasResults();
        expect(hasResults).toBe(true);
        // up-most should not
        await navbar.clickCalculatorTabForUid("ZTFxeW");
        hasResults = await calcPage.hasResults();
        expect(hasResults).toBe(false);
        await navbar.clickCalculatorTabForUid("Z3EwY2");
        // middle one should
        hasResults = await calcPage.hasResults();
        expect(hasResults).toBe(true);

        // 2. get up-most module
        await navbar.clickCalculatorTabForUid("ZTFxeW");

        // modify an input that is not linked
        await calcPage.getInputById("Ks").clear();
        await calcPage.getInputById("Ks").sendKeys("42");

        // other 2 modules should still have their results
        for (let i = 1; i < 3; i++) {
            await navbar.clickCalculatorTab(i);
            hasResults = await calcPage.hasResults();
            expect(hasResults).toBe(true);
        }

        // 3. get up-most module again
        await navbar.clickCalculatorTabForUid("ZTFxeW");

        // modify input that is linked
        await calcPage.getInputById("LargeurBerge").clear();
        await calcPage.getInputById("LargeurBerge").sendKeys("2.6");

        // check all 3 modules for absence of results
        for (let i = 0; i < 3; i++) {
            await navbar.clickCalculatorTab(i);
            hasResults = await calcPage.hasResults();
            expect(hasResults).toBe(false);
        }
    });

    it("when loading session-cascade-results.json, computation and results reset should be chained", async () => {
        // load session file
        await startPage.navigateTo();
        await navbar.clickMenuButton();
        await browser.sleep(200);
        await sidenav.clickLoadSessionButton();
        await browser.sleep(200);
        await sidenav.loadSessionFile("./session/session-cascade-results.json");
        await browser.sleep(500);
        expect(await navbar.getAllCalculatorTabs().count()).toBe(3);

        // dirty hack : get unwanted module first to prevent Calc button
        // of PAB Dimensions to be greyed-out @WTF
        await navbar.clickCalculatorTabForUid("Y2s1cG");
        // 1. get down-most module (PAB Dimensions)
        await navbar.clickCalculatorTabForUid("eHh5YX");

        // click "compute" button
        const calcButton = calcPage.getCalculateButton();
        await calcButton.click();

        // check all 3 modules for results
        for (let i = 0; i < 3; i++) {
            await navbar.clickCalculatorTab(i);
            const hasResults = await calcPage.hasResults();
            expect(hasResults).toBe(true);
        }

        // 2. get up-most module (Macro-rugo)
        await navbar.clickCalculatorTabForUid("MGg5OH");

        // modify any input (for ex. "Ks")
        await calcPage.getInputById("Ks").clear();
        await calcPage.getInputById("Ks").sendKeys("42");

        // check all 3 modules for absence of results
        for (let i = 0; i < 3; i++) {
            await navbar.clickCalculatorTab(i);
            const hasResults = await calcPage.hasResults();
            expect(hasResults).toBe(false);
        }
    });

    it("when loading session-results-invalidation.json, results reset should not be triggered on calculation", async () => {
        // disable evil option "empty fields on module creation"
        await prefPage.navigateTo();
        await prefPage.disableEvilEmptyFields();
        await browser.sleep(200);

        // start page
        await navbar.clickNewCalculatorButton();
        await browser.sleep(200);

        // load session file
        await navbar.clickMenuButton();
        await browser.sleep(200);
        await sidenav.clickLoadSessionButton();
        await browser.sleep(200);
        await sidenav.loadSessionFile("./session/session-results-invalidation.json");
        await browser.sleep(500);
        expect(await navbar.getAllCalculatorTabs().count()).toBe(2);

        // get down-most module (Ouvrages)
        await navbar.clickCalculatorTabForUid("amd2OG");

        // click "compute" button
        const calcButton = calcPage.getCalculateButton();
        await calcButton.click();
        const hasResults = await calcPage.hasResults();
        expect(hasResults).toBe(true);

        // 2. get up-most module (Ouvrages 1)
        await navbar.clickCalculatorTabForUid("aTgwMm");

        // modify an input that is not linked
        await calcPage.getInputById("Z2").clear();
        await calcPage.getInputById("Z2").sendKeys("101.8");

        // the down-most module should still have its results
        await navbar.clickCalculatorTabForUid("amd2OG");
        const hasResults2 = await calcPage.hasResults();
        expect(hasResults2).toBe(true);

        // calculate the upmost module
        await navbar.clickCalculatorTabForUid("aTgwMm");
        const calcButton2 = calcPage.getCalculateButton();
        await calcButton2.click();

        // the down-most module should still have its results
        await navbar.clickCalculatorTabForUid("amd2OG");
        const hasResults3 = await calcPage.hasResults();
        expect(hasResults3).toBe(true);

        // modify an input that is linked
        await navbar.clickCalculatorTabForUid("aTgwMm");
        await calcPage.getInputById("0_ZDV").clear();
        await calcPage.getInputById("0_ZDV").sendKeys("101");

        // the down-most module should not have its results anymore
        await navbar.clickCalculatorTabForUid("amd2OG");
        const hasResults4 = await calcPage.hasResults();
        expect(hasResults4).toBe(false);

    });

});
