import { by, element, protractor } from "protractor";

export class Navbar {
    getAllCalculatorTabs() {
        return element.all(by.css("#tabs-container button.calculator-button"));
    }

    /**
     * Robust method that returns number of calculator entries,
     * whether they are buttons or dropdown select options
     */
    async getCalculatorEntriesCount() {
        // if dropDown calculators select is visible
        const dropDown = element(by.css("mat-select#selectCalculator"));
        if (await dropDown.isPresent() && await dropDown.isDisplayed()) {
            await dropDown.click();
            const options = element.all(by.css(".cdk-overlay-container mat-option"));
            await dropDown.sendKeys(protractor.Key.ESCAPE); // close dropdown
            return await options.count();
        } else {
            return (await element.all(by.css("#tabs-container button.calculator-button"))).length;
        }
    }

    /**
     * Robust method that opens the nth calculator, whether the
     * calculator selector is a list of buttons or a dropdown select
     */
    async openNthCalculator(n: number) {
        // if dropDown calculators select is visible
        const dropDown = element(by.css("mat-select#selectCalculator"));
        if (await dropDown.isPresent() && await dropDown.isDisplayed()) {
            await dropDown.click();
            const optionId = ".cdk-overlay-container mat-option#mat-option-" + n;
            const option = element(by.css(optionId));
            await option.click();
        } else {
            const tabs = this.getAllCalculatorTabs();
            await tabs.get(n).click();
        }
    }

    getCalculatorTabForUid(uid: string) {
        return element(by.css("#tabs-container button.calculator-button.calculator-uid-" + uid));
    }

    getNewCalculatorButton() {
        return element(by.css("#new-calculator"));
    }

    getMenuButton() {
        return element(by.css("#open-menu"));
    }

    async clickCalculatorTab(n: number) {
        const tabs = this.getAllCalculatorTabs();
        await tabs.get(n).click();
    }

    async clickCalculatorTabForUid(uid: string) {
        const tab = this.getCalculatorTabForUid(uid);
        await tab.click();
    }

    async clickRandomCalculatorTab(n: number) {
        const tabs = this.getAllCalculatorTabs();
        const l = await tabs.count();
        const r = Math.min((Math.floor(Math.random() * l)), (l - 1));
        await tabs.get(r).click();
    }

    async clickNewCalculatorButton() {
        const ncb = this.getNewCalculatorButton();
        await ncb.click();
    }

    async clickMenuButton() {
        const ncb = this.getMenuButton();
        await ncb.click();
    }
}
