import { browser, by, element } from "protractor";
import { CalculatorPage } from "./calculator.po";
import { ListPage } from "./list.po";
import { Navbar } from "./navbar.po";
import { PreferencesPage } from "./preferences.po"

/**
 * check that fields are empty on creation
 */
describe("ngHyd − check that predam fields are empty on creation", () => {
    let listPage: ListPage;
    let prefPage: PreferencesPage;
    let navBar: Navbar;
    let calcPage: CalculatorPage;

    beforeEach(async () => {
        prefPage = new PreferencesPage();
        listPage = new ListPage();
        navBar = new Navbar();
        calcPage = new CalculatorPage();

        // enable evil option "empty fields on module creation"
        await prefPage.navigateTo();
        await browser.sleep(200);
        await prefPage.enableEvilEmptyFields();
        await browser.sleep(200);
    });

    it("when a basin is added", async () => {
        // open predam calculator
        await navBar.clickNewCalculatorButton();
        await listPage.clickMenuEntryForCalcType(30);
        await browser.sleep(200);

        // add basin
        const addBasinBtn = element(by.id("add-basin"));
        await addBasinBtn.click();
        await browser.sleep(200);

        // check "surface" input is empty
        let inp = calcPage.getInputById("3_S");
        let txt = await inp.getAttribute("value");
        expect(txt).toEqual("");

        // check "cote de fond" input is empty
        inp = calcPage.getInputById("3_ZF");
        txt = await inp.getAttribute("value");
        expect(txt).toEqual("");
    });

    it("when a wall is added", async () => {
        // open predam calculator
        await navBar.clickNewCalculatorButton();
        await listPage.clickMenuEntryForCalcType(30);
        await browser.sleep(200);

        // add wall
        const addWallBtn = element(by.id("add-wall"));
        await addWallBtn.click();

        // connect basins
        const connectBasinsBtn = element(by.id("validate-connect-basins"));
        await connectBasinsBtn.click();

        // check ZDV input is empty
        let inp = calcPage.getInputById("0_ZDV");
        let txt = await inp.getAttribute("value");
        expect(txt).toEqual("");

        // check L input is empty
        inp = calcPage.getInputById("0_L");
        txt = await inp.getAttribute("value");
        expect(txt).toEqual("");

        // check CdWSL input is empty (in this case, the structure happens to be a Larinier weir
        // which discharge coefficient must be empty)
        inp = calcPage.getInputById("0_CdWSL");
        txt = await inp.getAttribute("value");
        expect(txt).toEqual("");
    });
});
